@@ -1,15 +1,20 @@
-from pkg_resources import resource_stream
+from pkg_resources import (resource_stream, resource_listdir)
 import argparse
 import datetime
 import re
 import os
 import subprocess
 import sys
+import getpass
 
-__version__ = "0.2"
+__version__ = "0.3"
+
+LICENSES = []
+for file in sorted(resource_listdir(__name__, '.')):
+    match = re.match(r'template-([a-z0-9_]+).txt', file)
+    if match:
+        LICENSES.append(match.groups()[0])
 
-LICENSES = ["agpl3", "apache", "bsd2", "bsd3", "cddl", "cc0",
-            "epl", "gpl2", "gpl3", "lgpl", "mit", "mpl"]
 DEFAULT_LICENSE = "bsd3"
 
 
@@ -23,6 +28,14 @@ def clean_path(p):
     return p
 
 
+def get_context(args):
+    return {
+        "year": args.year,
+        "organization": args.organization,
+        "project": args.project,
+    }
+
+
 def guess_organization():
     """ Guess the organization from `git config`. If that can't be found,
         fall back to $USER environment variable.
@@ -30,9 +43,9 @@ def guess_organization():
     try:
         stdout = subprocess.check_output('git config --get user.name'.split())
         org = stdout.strip()
-    except OSError:
-        org = os.environ["USER"]
-    return org
+    except:
+        org = getpass.getuser()
+    return org.decode("UTF-8")
 
 
 def load_file_template(path):
@@ -45,10 +58,11 @@ def load_file_template(path):
     return template
 
 
-def load_package_template(license):
+def load_package_template(license, header=False):
     """ Load license template distributed with package.
     """
-    with resource_stream(__name__, 'template-%s.txt' % license) as licfile:
+    filename = 'template-%s-header.txt' if header else 'template-%s.txt'
+    with resource_stream(__name__, filename % license) as licfile:
         content = licfile.read()
     return content
 
@@ -76,17 +90,25 @@ def generate_license(template, context):
 
 def main():
 
+    def valid_year(string):
+        if not re.match(r"^\d{4}$", string):
+            raise argparse.ArgumentTypeError("Must be a four digit year")
+        return string
+
     parser = argparse.ArgumentParser(description='Generate a license')
 
-    parser.add_argument('license', metavar='license', nargs="?",
+    parser.add_argument('license', metavar='license', nargs="?", choices=LICENSES,
                        help='the license to generate, one of: %s' % ", ".join(LICENSES))
-    parser.add_argument('-o', '--org', dest='organization',
+    parser.add_argument('--header', dest='header', action="store_true",
+                       help='generate source file header for specified license')
+    parser.add_argument('-o', '--org', dest='organization', default=guess_organization(),
                        help='organization, defaults to .gitconfig or os.environ["USER"]')
-    parser.add_argument('-p', '--proj', dest='project',
+    parser.add_argument('-p', '--proj', dest='project', default=os.getcwd().split(os.sep)[-1],
                        help='name of project, defaults to name of current directory')
     parser.add_argument('-t', '--template', dest='template_path',
                        help='path to license template file')
-    parser.add_argument('-y', '--year', dest='year',
+    parser.add_argument('-y', '--year', dest='year', type=valid_year,
+                       default="%i" % datetime.date.today().year,
                        help='copyright year')
     parser.add_argument('--vars', dest='list_vars', action="store_true",
                        help='list template variables for specified license')
@@ -97,8 +119,23 @@ def main():
 
     license = args.license or DEFAULT_LICENSE
 
-    if license not in LICENSES:
-        parser.error("license must be one of: %s" % ", ".join(LICENSES))
+    # generate header if requested
+
+    if args.header:
+
+        if args.template_path:
+            template = load_file_template(args.template_path)
+        else:
+            try:
+                template = load_package_template(license, header=True)
+            except IOError:
+                sys.stderr.write("Sorry, no source headers are available for %s.\n" % args.license)
+                sys.exit(1)
+
+        content = generate_license(template.decode("UTF-8"), get_context(args))
+        sys.stdout.write(content)
+
+        sys.exit(0)
 
     # list template vars if requested
 
@@ -120,25 +157,14 @@ def main():
 
         sys.exit(0)
 
-    # check year, if specified
-
-    if args.year and not re.match(r"^\d{4}$", args.year):
-        parser.error("-y must be a four digit year")
-
     # create context
 
-    context = {
-        "year": args.year or "%i" % datetime.date.today().year,
-        "organization": args.organization or guess_organization(),
-        "project": args.project or os.getcwd().split(os.sep)[-1],
-    }
-
     if args.template_path:
         template = load_file_template(args.template_path)
     else:
         template = load_package_template(license)
 
-    content = generate_license(template, context)
+    content = generate_license(template.decode("UTF-8"), get_context(args))
     sys.stdout.write(content)
 
 
